@@ -15,4 +15,6 @@
 // specific language governing permissions and limitations
 // under the License.
 
+mod coop;
+mod datasource_split;
 mod logical_plan;
