@@ -51,7 +51,7 @@ use datafusion_physical_plan::joins::{
     HashJoinExec, NestedLoopJoinExec, PartitionMode, StreamJoinPartitionMode,
     SymmetricHashJoinExec,
 };
-use datafusion_physical_plan::projection::{update_expr, ProjectionExec};
+use datafusion_physical_plan::projection::{update_expr, ProjectionExec, ProjectionExpr};
 use datafusion_physical_plan::repartition::RepartitionExec;
 use datafusion_physical_plan::sorts::sort::SortExec;
 use datafusion_physical_plan::sorts::sort_preserving_merge::SortPreservingMergeExec;
@@ -63,7 +63,7 @@ use insta::assert_snapshot;
 use itertools::Itertools;
 
 /// Mocked UDF
-#[derive(Debug)]
+#[derive(Debug, PartialEq, Eq, Hash)]
 struct DummyUDF {
     signature: Signature,
 }
@@ -128,6 +128,7 @@ fn test_update_matching_exprs() -> Result<()> {
                 )),
             ],
             Field::new("f", DataType::Int32, true).into(),
+            Arc::new(ConfigOptions::default()),
         )),
         Arc::new(CaseExpr::try_new(
             Some(Arc::new(Column::new("d", 2))),
@@ -193,6 +194,7 @@ fn test_update_matching_exprs() -> Result<()> {
                 )),
             ],
             Field::new("f", DataType::Int32, true).into(),
+            Arc::new(ConfigOptions::default()),
         )),
         Arc::new(CaseExpr::try_new(
             Some(Arc::new(Column::new("d", 3))),
@@ -222,8 +224,12 @@ fn test_update_matching_exprs() -> Result<()> {
         )?),
     ];
 
+    let child_exprs: Vec<ProjectionExpr> = child
+        .iter()
+        .map(|(expr, alias)| ProjectionExpr::new(expr.clone(), alias.clone()))
+        .collect();
     for (expr, expected_expr) in exprs.into_iter().zip(expected_exprs.into_iter()) {
-        assert!(update_expr(&expr, &child, true)?
+        assert!(update_expr(&expr, &child_exprs, true)?
             .unwrap()
             .eq(&expected_expr));
     }
@@ -261,6 +267,7 @@ fn test_update_projected_exprs() -> Result<()> {
                 )),
             ],
             Field::new("f", DataType::Int32, true).into(),
+            Arc::new(ConfigOptions::default()),
         )),
         Arc::new(CaseExpr::try_new(
             Some(Arc::new(Column::new("d", 2))),
@@ -326,6 +333,7 @@ fn test_update_projected_exprs() -> Result<()> {
                 )),
             ],
             Field::new("f", DataType::Int32, true).into(),
+            Arc::new(ConfigOptions::default()),
         )),
         Arc::new(CaseExpr::try_new(
             Some(Arc::new(Column::new("d_new", 3))),
@@ -355,8 +363,12 @@ fn test_update_projected_exprs() -> Result<()> {
         )?),
     ];
 
+    let proj_exprs: Vec<ProjectionExpr> = projected_exprs
+        .iter()
+        .map(|(expr, alias)| ProjectionExpr::new(expr.clone(), alias.clone()))
+        .collect();
     for (expr, expected_expr) in exprs.into_iter().zip(expected_exprs.into_iter()) {
-        assert!(update_expr(&expr, &projected_exprs, false)?
+        assert!(update_expr(&expr, &proj_exprs, false)?
             .unwrap()
             .eq(&expected_expr));
     }
@@ -420,8 +432,8 @@ fn test_csv_after_projection() -> Result<()> {
     let csv = create_projecting_csv_exec();
     let projection: Arc<dyn ExecutionPlan> = Arc::new(ProjectionExec::try_new(
         vec![
-            (Arc::new(Column::new("b", 2)), "b".to_string()),
-            (Arc::new(Column::new("d", 0)), "d".to_string()),
+            ProjectionExpr::new(Arc::new(Column::new("b", 2)), "b".to_string()),
+            ProjectionExpr::new(Arc::new(Column::new("d", 0)), "d".to_string()),
         ],
         csv.clone(),
     )?);
@@ -457,9 +469,9 @@ fn test_memory_after_projection() -> Result<()> {
     let memory = create_projecting_memory_exec();
     let projection: Arc<dyn ExecutionPlan> = Arc::new(ProjectionExec::try_new(
         vec![
-            (Arc::new(Column::new("d", 2)), "d".to_string()),
-            (Arc::new(Column::new("e", 3)), "e".to_string()),
-            (Arc::new(Column::new("a", 1)), "a".to_string()),
+            ProjectionExpr::new(Arc::new(Column::new("d", 2)), "d".to_string()),
+            ProjectionExpr::new(Arc::new(Column::new("e", 3)), "e".to_string()),
+            ProjectionExpr::new(Arc::new(Column::new("a", 1)), "a".to_string()),
         ],
         memory.clone(),
     )?);
@@ -563,9 +575,9 @@ fn test_streaming_table_after_projection() -> Result<()> {
     )?;
     let projection = Arc::new(ProjectionExec::try_new(
         vec![
-            (Arc::new(Column::new("d", 3)), "d".to_string()),
-            (Arc::new(Column::new("e", 2)), "e".to_string()),
-            (Arc::new(Column::new("a", 0)), "a".to_string()),
+            ProjectionExpr::new(Arc::new(Column::new("d", 3)), "d".to_string()),
+            ProjectionExpr::new(Arc::new(Column::new("e", 2)), "e".to_string()),
+            ProjectionExpr::new(Arc::new(Column::new("a", 0)), "a".to_string()),
         ],
         Arc::new(streaming_table) as _,
     )?) as _;
@@ -630,17 +642,17 @@ fn test_projection_after_projection() -> Result<()> {
     let csv = create_simple_csv_exec();
     let child_projection: Arc<dyn ExecutionPlan> = Arc::new(ProjectionExec::try_new(
         vec![
-            (Arc::new(Column::new("c", 2)), "c".to_string()),
-            (Arc::new(Column::new("e", 4)), "new_e".to_string()),
-            (Arc::new(Column::new("a", 0)), "a".to_string()),
-            (Arc::new(Column::new("b", 1)), "new_b".to_string()),
+            ProjectionExpr::new(Arc::new(Column::new("c", 2)), "c".to_string()),
+            ProjectionExpr::new(Arc::new(Column::new("e", 4)), "new_e".to_string()),
+            ProjectionExpr::new(Arc::new(Column::new("a", 0)), "a".to_string()),
+            ProjectionExpr::new(Arc::new(Column::new("b", 1)), "new_b".to_string()),
         ],
         csv.clone(),
     )?);
     let top_projection: Arc<dyn ExecutionPlan> = Arc::new(ProjectionExec::try_new(
         vec![
-            (Arc::new(Column::new("new_b", 3)), "new_b".to_string()),
-            (
+            ProjectionExpr::new(Arc::new(Column::new("new_b", 3)), "new_b".to_string()),
+            ProjectionExpr::new(
                 Arc::new(BinaryExpr::new(
                     Arc::new(Column::new("c", 0)),
                     Operator::Plus,
@@ -648,7 +660,10 @@ fn test_projection_after_projection() -> Result<()> {
                 )),
                 "binary".to_string(),
             ),
-            (Arc::new(Column::new("new_b", 3)), "newest_b".to_string()),
+            ProjectionExpr::new(
+                Arc::new(Column::new("new_b", 3)),
+                "newest_b".to_string(),
+            ),
         ],
         child_projection.clone(),
     )?);
@@ -712,12 +727,13 @@ fn test_output_req_after_projection() -> Result<()> {
             Arc::new(Column::new("a", 0)),
             Arc::new(Column::new("b", 1)),
         ]),
+        None,
     ));
     let projection: Arc<dyn ExecutionPlan> = Arc::new(ProjectionExec::try_new(
         vec![
-            (Arc::new(Column::new("c", 2)), "c".to_string()),
-            (Arc::new(Column::new("a", 0)), "new_a".to_string()),
-            (Arc::new(Column::new("b", 1)), "b".to_string()),
+            ProjectionExpr::new(Arc::new(Column::new("c", 2)), "c".to_string()),
+            ProjectionExpr::new(Arc::new(Column::new("a", 0)), "new_a".to_string()),
+            ProjectionExpr::new(Arc::new(Column::new("b", 1)), "b".to_string()),
         ],
         sort_req.clone(),
     )?);
@@ -807,9 +823,9 @@ fn test_coalesce_partitions_after_projection() -> Result<()> {
         Arc::new(CoalescePartitionsExec::new(csv));
     let projection: Arc<dyn ExecutionPlan> = Arc::new(ProjectionExec::try_new(
         vec![
-            (Arc::new(Column::new("b", 1)), "b".to_string()),
-            (Arc::new(Column::new("a", 0)), "a_new".to_string()),
-            (Arc::new(Column::new("d", 3)), "d".to_string()),
+            ProjectionExpr::new(Arc::new(Column::new("b", 1)), "b".to_string()),
+            ProjectionExpr::new(Arc::new(Column::new("a", 0)), "a_new".to_string()),
+            ProjectionExpr::new(Arc::new(Column::new("d", 3)), "d".to_string()),
         ],
         coalesce_partitions,
     )?);
@@ -864,9 +880,9 @@ fn test_filter_after_projection() -> Result<()> {
     let filter = Arc::new(FilterExec::try_new(predicate, csv)?);
     let projection: Arc<dyn ExecutionPlan> = Arc::new(ProjectionExec::try_new(
         vec![
-            (Arc::new(Column::new("a", 0)), "a_new".to_string()),
-            (Arc::new(Column::new("b", 1)), "b".to_string()),
-            (Arc::new(Column::new("d", 3)), "d".to_string()),
+            ProjectionExpr::new(Arc::new(Column::new("a", 0)), "a_new".to_string()),
+            ProjectionExpr::new(Arc::new(Column::new("b", 1)), "b".to_string()),
+            ProjectionExpr::new(Arc::new(Column::new("d", 3)), "d".to_string()),
         ],
         filter.clone(),
     )?) as _;
@@ -959,11 +975,17 @@ fn test_join_after_projection() -> Result<()> {
     )?);
     let projection: Arc<dyn ExecutionPlan> = Arc::new(ProjectionExec::try_new(
         vec![
-            (Arc::new(Column::new("c", 2)), "c_from_left".to_string()),
-            (Arc::new(Column::new("b", 1)), "b_from_left".to_string()),
-            (Arc::new(Column::new("a", 0)), "a_from_left".to_string()),
-            (Arc::new(Column::new("a", 5)), "a_from_right".to_string()),
-            (Arc::new(Column::new("c", 7)), "c_from_right".to_string()),
+            ProjectionExpr::new(Arc::new(Column::new("c", 2)), "c_from_left".to_string()),
+            ProjectionExpr::new(Arc::new(Column::new("b", 1)), "b_from_left".to_string()),
+            ProjectionExpr::new(Arc::new(Column::new("a", 0)), "a_from_left".to_string()),
+            ProjectionExpr::new(
+                Arc::new(Column::new("a", 5)),
+                "a_from_right".to_string(),
+            ),
+            ProjectionExpr::new(
+                Arc::new(Column::new("c", 7)),
+                "c_from_right".to_string(),
+            ),
         ],
         join,
     )?) as _;
@@ -1084,16 +1106,16 @@ fn test_join_after_required_projection() -> Result<()> {
     )?);
     let projection: Arc<dyn ExecutionPlan> = Arc::new(ProjectionExec::try_new(
         vec![
-            (Arc::new(Column::new("a", 5)), "a".to_string()),
-            (Arc::new(Column::new("b", 6)), "b".to_string()),
-            (Arc::new(Column::new("c", 7)), "c".to_string()),
-            (Arc::new(Column::new("d", 8)), "d".to_string()),
-            (Arc::new(Column::new("e", 9)), "e".to_string()),
-            (Arc::new(Column::new("a", 0)), "a".to_string()),
-            (Arc::new(Column::new("b", 1)), "b".to_string()),
-            (Arc::new(Column::new("c", 2)), "c".to_string()),
-            (Arc::new(Column::new("d", 3)), "d".to_string()),
-            (Arc::new(Column::new("e", 4)), "e".to_string()),
+            ProjectionExpr::new(Arc::new(Column::new("a", 5)), "a".to_string()),
+            ProjectionExpr::new(Arc::new(Column::new("b", 6)), "b".to_string()),
+            ProjectionExpr::new(Arc::new(Column::new("c", 7)), "c".to_string()),
+            ProjectionExpr::new(Arc::new(Column::new("d", 8)), "d".to_string()),
+            ProjectionExpr::new(Arc::new(Column::new("e", 9)), "e".to_string()),
+            ProjectionExpr::new(Arc::new(Column::new("a", 0)), "a".to_string()),
+            ProjectionExpr::new(Arc::new(Column::new("b", 1)), "b".to_string()),
+            ProjectionExpr::new(Arc::new(Column::new("c", 2)), "c".to_string()),
+            ProjectionExpr::new(Arc::new(Column::new("d", 3)), "d".to_string()),
+            ProjectionExpr::new(Arc::new(Column::new("e", 4)), "e".to_string()),
         ],
         join,
     )?) as _;
@@ -1173,7 +1195,7 @@ fn test_nested_loop_join_after_projection() -> Result<()> {
     )?) as _;
 
     let projection: Arc<dyn ExecutionPlan> = Arc::new(ProjectionExec::try_new(
-        vec![(col_left_c, "c".to_string())],
+        vec![ProjectionExpr::new(col_left_c, "c".to_string())],
         Arc::clone(&join),
     )?) as _;
     let initial = displayable(projection.as_ref()).indent(true).to_string();
@@ -1263,10 +1285,13 @@ fn test_hash_join_after_projection() -> Result<()> {
     )?);
     let projection: Arc<dyn ExecutionPlan> = Arc::new(ProjectionExec::try_new(
         vec![
-            (Arc::new(Column::new("c", 2)), "c_from_left".to_string()),
-            (Arc::new(Column::new("b", 1)), "b_from_left".to_string()),
-            (Arc::new(Column::new("a", 0)), "a_from_left".to_string()),
-            (Arc::new(Column::new("c", 7)), "c_from_right".to_string()),
+            ProjectionExpr::new(Arc::new(Column::new("c", 2)), "c_from_left".to_string()),
+            ProjectionExpr::new(Arc::new(Column::new("b", 1)), "b_from_left".to_string()),
+            ProjectionExpr::new(Arc::new(Column::new("a", 0)), "a_from_left".to_string()),
+            ProjectionExpr::new(
+                Arc::new(Column::new("c", 7)),
+                "c_from_right".to_string(),
+            ),
         ],
         join.clone(),
     )?) as _;
@@ -1302,10 +1327,10 @@ fn test_hash_join_after_projection() -> Result<()> {
 
     let projection = Arc::new(ProjectionExec::try_new(
         vec![
-            (Arc::new(Column::new("a", 0)), "a".to_string()),
-            (Arc::new(Column::new("b", 1)), "b".to_string()),
-            (Arc::new(Column::new("c", 2)), "c".to_string()),
-            (Arc::new(Column::new("c", 7)), "c".to_string()),
+            ProjectionExpr::new(Arc::new(Column::new("a", 0)), "a".to_string()),
+            ProjectionExpr::new(Arc::new(Column::new("b", 1)), "b".to_string()),
+            ProjectionExpr::new(Arc::new(Column::new("c", 2)), "c".to_string()),
+            ProjectionExpr::new(Arc::new(Column::new("c", 7)), "c".to_string()),
         ],
         join.clone(),
     )?);
@@ -1346,9 +1371,9 @@ fn test_repartition_after_projection() -> Result<()> {
     )?);
     let projection: Arc<dyn ExecutionPlan> = Arc::new(ProjectionExec::try_new(
         vec![
-            (Arc::new(Column::new("b", 1)), "b_new".to_string()),
-            (Arc::new(Column::new("a", 0)), "a".to_string()),
-            (Arc::new(Column::new("d", 3)), "d_new".to_string()),
+            ProjectionExpr::new(Arc::new(Column::new("b", 1)), "b_new".to_string()),
+            ProjectionExpr::new(Arc::new(Column::new("a", 0)), "a".to_string()),
+            ProjectionExpr::new(Arc::new(Column::new("d", 3)), "d_new".to_string()),
         ],
         repartition,
     )?) as _;
@@ -1416,9 +1441,9 @@ fn test_sort_after_projection() -> Result<()> {
     );
     let projection: Arc<dyn ExecutionPlan> = Arc::new(ProjectionExec::try_new(
         vec![
-            (Arc::new(Column::new("c", 2)), "c".to_string()),
-            (Arc::new(Column::new("a", 0)), "new_a".to_string()),
-            (Arc::new(Column::new("b", 1)), "b".to_string()),
+            ProjectionExpr::new(Arc::new(Column::new("c", 2)), "c".to_string()),
+            ProjectionExpr::new(Arc::new(Column::new("a", 0)), "new_a".to_string()),
+            ProjectionExpr::new(Arc::new(Column::new("b", 1)), "b".to_string()),
         ],
         Arc::new(sort_exec),
     )?) as _;
@@ -1470,9 +1495,9 @@ fn test_sort_preserving_after_projection() -> Result<()> {
     );
     let projection: Arc<dyn ExecutionPlan> = Arc::new(ProjectionExec::try_new(
         vec![
-            (Arc::new(Column::new("c", 2)), "c".to_string()),
-            (Arc::new(Column::new("a", 0)), "new_a".to_string()),
-            (Arc::new(Column::new("b", 1)), "b".to_string()),
+            ProjectionExpr::new(Arc::new(Column::new("c", 2)), "c".to_string()),
+            ProjectionExpr::new(Arc::new(Column::new("a", 0)), "new_a".to_string()),
+            ProjectionExpr::new(Arc::new(Column::new("b", 1)), "b".to_string()),
         ],
         Arc::new(sort_exec),
     )?) as _;
@@ -1513,9 +1538,9 @@ fn test_union_after_projection() -> Result<()> {
     let union = Arc::new(UnionExec::new(vec![csv.clone(), csv.clone(), csv]));
     let projection: Arc<dyn ExecutionPlan> = Arc::new(ProjectionExec::try_new(
         vec![
-            (Arc::new(Column::new("c", 2)), "c".to_string()),
-            (Arc::new(Column::new("a", 0)), "new_a".to_string()),
-            (Arc::new(Column::new("b", 1)), "b".to_string()),
+            ProjectionExpr::new(Arc::new(Column::new("c", 2)), "c".to_string()),
+            ProjectionExpr::new(Arc::new(Column::new("a", 0)), "new_a".to_string()),
+            ProjectionExpr::new(Arc::new(Column::new("b", 1)), "b".to_string()),
         ],
         union.clone(),
     )?) as _;
@@ -1584,15 +1609,15 @@ fn test_partition_col_projection_pushdown() -> Result<()> {
 
     let projection: Arc<dyn ExecutionPlan> = Arc::new(ProjectionExec::try_new(
         vec![
-            (
+            ProjectionExpr::new(
                 col("string_col", partitioned_schema.as_ref())?,
                 "string_col".to_string(),
             ),
-            (
+            ProjectionExpr::new(
                 col("partition_col", partitioned_schema.as_ref())?,
                 "partition_col".to_string(),
             ),
-            (
+            ProjectionExpr::new(
                 col("int_col", partitioned_schema.as_ref())?,
                 "int_col".to_string(),
             ),
@@ -1625,11 +1650,11 @@ fn test_partition_col_projection_pushdown_expr() -> Result<()> {
 
     let projection: Arc<dyn ExecutionPlan> = Arc::new(ProjectionExec::try_new(
         vec![
-            (
+            ProjectionExpr::new(
                 col("string_col", partitioned_schema.as_ref())?,
                 "string_col".to_string(),
             ),
-            (
+            ProjectionExpr::new(
                 // CAST(partition_col, Utf8View)
                 cast(
                     col("partition_col", partitioned_schema.as_ref())?,
@@ -1638,7 +1663,7 @@ fn test_partition_col_projection_pushdown_expr() -> Result<()> {
                 )?,
                 "partition_col".to_string(),
             ),
-            (
+            ProjectionExpr::new(
                 col("int_col", partitioned_schema.as_ref())?,
                 "int_col".to_string(),
             ),
