@@ -33,7 +33,7 @@ use std::sync::Arc;
     syntax_example = "isnan(numeric_expression)",
     standard_argument(name = "numeric_expression", prefix = "Numeric")
 )]
-#[derive(Debug)]
+#[derive(Debug, PartialEq, Eq, Hash)]
 pub struct IsNanFunc {
     signature: Signature,
 }
